@@ -77,34 +77,44 @@ between Python C API code and primer3 native C code.
 // It must not be changed or freed, so we have to malloc new memory for the
 // param value.
 #if PY_MAJOR_VERSION < 3
-    #define DICT_GET_AND_COPY_STR(o, d, k, st)                                 \
+    #define DICT_GET_AND_COPY_STR(o, d, k, st, tc, ss)                         \
         if (DICT_GET_OBJ(o, d, k)) {                                           \
             if (!PyString_Check(o)){                                           \
                 PyErr_Format(PyExc_TypeError,                                  \
                              "Value of %s is not of type string", k);          \
                 return NULL;}                                                  \
-            *st = (char *) malloc(PyBytes_Size(o));                            \
+            if (PyString_AsStringAndSize(o, &tc, &ss) == -1) {                 \
+                return NULL;}                                                  \
+            *st = (char *) malloc((ss + 1) * sizeof(char));                    \
             if (st == NULL) {                                                  \
                 PyErr_Format(PyExc_IOError,                                    \
                             "Could not allocate memory while copying %s", k);  \
                 return NULL;}                                                  \
-            strcpy(*st, PyString_AsString(o));                                 \
+            memcpy(*st, tc, (int)(ss + 1));                                    \
         }
 #else
-    #define DICT_GET_AND_COPY_STR(o, d, k, st)                                 \
+    #define DICT_GET_AND_COPY_STR(o, d, k, st, tc, ss)                         \
         if (DICT_GET_OBJ(o, d, k)) {                                           \
             if (PyUnicode_Check(o)) {                                          \
-                o = PyUnicode_AsASCIIString(o);                                \
-            } else if (!PyBytes_Check(o)){                                     \
+                tc = PyUnicode_AsUTF8AndSize(o, &ss);                          \
+            } else if (PyBytes_Check(o)){                                      \
+                if (PyBytes_AsStringAndSize(o, &tc, &ss) == -1) {              \
+                    return NULL;}                                              \
+            } else {                                                           \
                 PyErr_Format(PyExc_TypeError,                                  \
                             "Value of %s is not of type unicode or bytes", k); \
                 return NULL;}                                                  \
-            *st = (char *) malloc(PyBytes_Size(o));                            \
+            if (tc == NULL){                                                   \
+                    PyErr_Format(PyExc_TypeError,                              \
+                            "Error processing string in %s", k);               \
+                    return NULL;                                               \
+                }                                                              \
+            *st = (char *) malloc(ss * sizeof(char));                          \
             if (st == NULL) {                                                  \
                 PyErr_Format(PyExc_IOError,                                    \
                             "Could not allocate memory while copying %s", k);  \
                 return NULL;}                                                  \
-            strcpy(*st, PyBytes_AsString(o));                                  \
+            memcpy(*st, tc, (int)(ss + 1));                                    \
         }
 #endif
 
@@ -121,7 +131,7 @@ between Python C API code and primer3 native C code.
             arr[i] = (int)PyLong_AsLong(PyList_GetItem(o, i));                 \
         }                                                                      \
         *st = arr;                                                             \
-    }
+    }                                                                          \
 
 #define DICT_GET_AND_COPY_ARRAY_INTO_ARRAY(o, d, k, st, arr_len)               \
     if (DICT_GET_OBJ(o, d, k)) {                                               \
@@ -134,7 +144,7 @@ between Python C API code and primer3 native C code.
         for (i=0; i < *arr_len; i++) {                                         \
             *st[i] = (int)PyLong_AsLong(PyList_GetItem(o, i));                 \
         }                                                                      \
-    }
+    }                                                                          \
 
 #define DICT_GET_AND_COPY_TO_INTERVAL_ARRAY(o, d, k, st)                       \
     if (DICT_GET_OBJ(o, d, k)){                                                \
@@ -156,54 +166,6 @@ between Python C API code and primer3 native C code.
         }                                                                      \
     }                                                                          \
 
-#if PY_MAJOR_VERSION < 3
-    #define COPY_SEQ_NAME_AND_SEQ_TO_SEQ_LIB(py_seq_name, py_seq, seq_lib) {   \
-        if (PyString_Check(py_seq_name)) {                                     \
-            seq_name = PyString_AsString(py_seq_name);                         \
-        } else {                                                               \
-            PyErr_SetString(PyExc_TypeError,                                   \
-                "Cannot add seq name with non-String type to seq_lib");        \
-            return NULL;                                                       \
-        }                                                                      \
-        if (PyString_Check(py_seq)) {                                          \
-            seq_name = PyString_AsString(py_seq);                              \
-        } else {                                                               \
-            PyErr_SetString(PyExc_TypeError,                                   \
-                "Cannot add seq with non-String type to seq_lib");             \
-            return NULL;                                                       \
-        }                                                                      \
-        if (add_seq_and_rev_comp_to_seq_lib(sl, seq, seq_name, errfrag)) {     \
-            PyErr_SetString(PyExc_IOError, errfrag);                           \
-            return NULL;                                                       \
-        }                                                                      \
-    }
-#else
-    #define COPY_SEQ_NAME_AND_SEQ_TO_SEQ_LIB(py_seq_name, py_seq, seq_lib) {   \
-        if (PyUnicode_Check(py_seq_name)) {                                    \
-            seq_name = PyBytes_AsString(PyUnicode_AsASCIIString(py_seq_name)); \
-        } else if (PyBytes_Check(py_seq_name)){                                \
-            seq_name = PyBytes_AsString(py_seq_name);                          \
-        } else {                                                               \
-            PyErr_SetString(PyExc_TypeError,                                   \
-                "Cannot add seq name with non-Unicode/Bytes type to seq_lib"); \
-            return NULL;                                                       \
-        }                                                                      \
-        if (PyUnicode_Check(py_seq)) {                                         \
-            seq = PyBytes_AsString(PyUnicode_AsASCIIString(py_seq));           \
-        } else if (PyBytes_Check(py_seq)){                                     \
-            seq = PyBytes_AsString(py_seq);                                    \
-        } else {                                                               \
-            PyErr_SetString(PyExc_TypeError,                                   \
-                "Cannot add seq with non-Unicode/Bytes type to seq_lib");      \
-            return NULL;                                                       \
-        }                                                                      \
-        if (add_seq_and_rev_comp_to_seq_lib(sl, seq, seq_name, errfrag)) {     \
-            PyErr_SetString(PyExc_IOError, errfrag);                           \
-            return NULL;                                                       \
-        }                                                                      \
-    }
-#endif
-
 
 p3_global_settings*
 _setGlobals(PyObject *p3s_dict) {
@@ -219,7 +181,8 @@ _setGlobals(PyObject *p3s_dict) {
     p3_global_settings      *pa;
     PyObject                *p_obj, *p_obj2, *p_obj3, *p_obj4;
     int                     i;
-    char                    *task_tmp=NULL;
+    Py_ssize_t              str_size;
+    char                    *temp_char, *task_tmp=NULL;
 
     if (!(pa = p3_create_global_settings())) {
         PyErr_SetString(PyExc_IOError, "Could not allocate memory for p3 globals");
@@ -333,10 +296,10 @@ _setGlobals(PyObject *p3s_dict) {
     DICT_GET_AND_ASSIGN_INT(p_obj, p3s_dict, "PRIMER_LOWERCASE_MASKING", pa->lowercase_masking);
     DICT_GET_AND_ASSIGN_INT(p_obj, p3s_dict, "PRIMER_THERMODYNAMIC_OLIGO_ALIGNMENT", pa->thermodynamic_oligo_alignment);
     DICT_GET_AND_ASSIGN_INT(p_obj, p3s_dict, "PRIMER_THERMODYNAMIC_TEMPLATE_ALIGNMENT", pa->thermodynamic_template_alignment);
-    DICT_GET_AND_COPY_STR(p_obj, p3s_dict, "PRIMER_MUST_MATCH_FIVE_PRIME", &pa->p_args.must_match_five_prime);
-    DICT_GET_AND_COPY_STR(p_obj, p3s_dict, "PRIMER_MUST_MATCH_THREE_PRIME", &pa->p_args.must_match_three_prime);
-    DICT_GET_AND_COPY_STR(p_obj, p3s_dict, "PRIMER_INTERNAL_MUST_MATCH_FIVE_PRIME", &pa->o_args.must_match_five_prime);
-    DICT_GET_AND_COPY_STR(p_obj, p3s_dict, "PRIMER_INTERNAL_MUST_MATCH_THREE_PRIME", &pa->o_args.must_match_three_prime);
+    DICT_GET_AND_COPY_STR(p_obj, p3s_dict, "PRIMER_MUST_MATCH_FIVE_PRIME", &pa->p_args.must_match_five_prime, temp_char, str_size);
+    DICT_GET_AND_COPY_STR(p_obj, p3s_dict, "PRIMER_MUST_MATCH_THREE_PRIME", &pa->p_args.must_match_three_prime, temp_char, str_size);
+    DICT_GET_AND_COPY_STR(p_obj, p3s_dict, "PRIMER_INTERNAL_MUST_MATCH_FIVE_PRIME", &pa->o_args.must_match_five_prime, temp_char, str_size);
+    DICT_GET_AND_COPY_STR(p_obj, p3s_dict, "PRIMER_INTERNAL_MUST_MATCH_THREE_PRIME", &pa->o_args.must_match_three_prime, temp_char, str_size);
     DICT_GET_AND_ASSIGN_DOUBLE(p_obj, p3s_dict, "PRIMER_WT_TM_GT", pa->p_args.weights.temp_gt);
     DICT_GET_AND_ASSIGN_DOUBLE(p_obj, p3s_dict, "PRIMER_WT_TM_LT", pa->p_args.weights.temp_lt);
     DICT_GET_AND_ASSIGN_DOUBLE(p_obj, p3s_dict, "PRIMER_WT_GC_PERCENT_GT", pa->p_args.weights.gc_content_gt);
@@ -430,7 +393,7 @@ _setGlobals(PyObject *p3s_dict) {
     }
 
     // Handler primer task
-    DICT_GET_AND_COPY_STR(p_obj, p3s_dict, "PRIMER_TASK", &task_tmp);
+    DICT_GET_AND_COPY_STR(p_obj, p3s_dict, "PRIMER_TASK", &task_tmp, temp_char, str_size);
     // if (task_tmp == NULL) {
     //     PyErr_SetString(PyExc_ValueError, \
     //         "Primer3 args must include PRIMER_TASK");
@@ -510,7 +473,49 @@ createSeqLib(PyObject *seq_dict){
 
     pos = 0;
     while (PyDict_Next(seq_dict, &pos, &py_seq_name, &py_seq)) {
-        COPY_SEQ_NAME_AND_SEQ_TO_SEQ_LIB(py_seq_name, py_seq, sl);
+#if PY_MAJOR_VERSION < 3 
+            if (PyString_Check(py_seq_name)) {                                    
+                seq_name = PyString_AsString(py_seq_name);                        
+            } else {                                                              
+                PyErr_SetString(PyExc_TypeError,                                  
+                    "Cannot add seq name with non-String type to seq_lib");       
+                return NULL;                                                      
+            }                                                                     
+            if (PyString_Check(py_seq)) {                                         
+                seq_name = PyString_AsString(py_seq);                             
+            } else {                                                              
+                PyErr_SetString(PyExc_TypeError,                                  
+                    "Cannot add seq with non-String type to seq_lib");            
+                return NULL;                                                      
+            }                                                                     
+            if (add_seq_and_rev_comp_to_seq_lib(sl, seq, seq_name, errfrag)) {    
+                PyErr_SetString(PyExc_IOError, errfrag);                          
+                return NULL;         
+            }                                                                                                              
+#else 
+            if (PyUnicode_Check(py_seq_name)) {                                   
+                seq_name = PyUnicode_AsUTF8(py_seq_name);
+            } else if (PyBytes_Check(py_seq_name)){                               
+                seq_name = PyBytes_AsString(py_seq_name);                         
+            } else {                                                              
+                PyErr_SetString(PyExc_TypeError,                                  
+                    "Cannot add seq name with non-Unicode/Bytes type to seq_lib");
+                return NULL;                                                      
+            }                                                                     
+            if (PyUnicode_Check(py_seq)) {                                        
+                seq = PyUnicode_AsUTF8(py_seq);          
+            } else if (PyBytes_Check(py_seq)){                                    
+                seq = PyBytes_AsString(py_seq);                                   
+            } else {                                                              
+                PyErr_SetString(PyExc_TypeError,                                  
+                    "Cannot add seq with non-Unicode/Bytes type to seq_lib");     
+                return NULL;                                                      
+            }                                                                     
+            if (add_seq_and_rev_comp_to_seq_lib(sl, seq, seq_name, errfrag)) {    
+                PyErr_SetString(PyExc_IOError, errfrag);                          
+                return NULL;                                                      
+            }                                                                     
+#endif
     }
     return sl;
 }
@@ -525,68 +530,83 @@ _setSeqArgs(PyObject *sa_dict, p3_global_settings *pa){
 
     seq_args                *sa;
     PyObject                *p_obj, *p_obj2, *p_obj3, *p_obj4;
-    int                     i, j, *arr_len=NULL;
-    int                     len1, len2;
-    int                     *seq_qual_len=NULL, *overlap_junction_len=NULL;
+    char                    *temp_char;
+    int                     i, j, len1, len2, *arr_len=NULL, *seq_qual_len=NULL;
+    Py_ssize_t              str_size;
 
     if (!(sa = create_seq_arg())) {
         PyErr_SetString(PyExc_IOError, "Could not allocate memory for seq_args");
         return NULL;
     }
-    DICT_GET_AND_COPY_STR(p_obj, sa_dict, "SEQUENCE_TEMPLATE", &sa->sequence);
-    DICT_GET_AND_COPY_STR(p_obj, sa_dict, "SEQUENCE_ID", &sa->sequence_name);
-    DICT_GET_AND_COPY_STR(p_obj, sa_dict, "SEQUENCE_PRIMER", &sa->left_input);
-    DICT_GET_AND_COPY_STR(p_obj, sa_dict, "SEQUENCE_PRIMER_REVCOMP", &sa->right_input);
-    DICT_GET_AND_COPY_STR(p_obj, sa_dict, "SEQUENCE_INTERNAL_OLIGO", &sa->internal_input);
+    DICT_GET_AND_COPY_STR(p_obj, sa_dict, "SEQUENCE_TEMPLATE", &sa->sequence, temp_char, str_size);
+    DICT_GET_AND_COPY_STR(p_obj, sa_dict, "SEQUENCE_ID", &sa->sequence_name, temp_char, str_size);
+    DICT_GET_AND_COPY_STR(p_obj, sa_dict, "SEQUENCE_PRIMER", &sa->left_input, temp_char, str_size);
+    DICT_GET_AND_COPY_STR(p_obj, sa_dict, "SEQUENCE_PRIMER_REVCOMP", &sa->right_input, temp_char, str_size);
+    DICT_GET_AND_COPY_STR(p_obj, sa_dict, "SEQUENCE_INTERNAL_OLIGO", &sa->internal_input, temp_char, str_size);
     DICT_GET_AND_COPY_ARRAY(p_obj, sa_dict, "SEQUENCE_QUALITY", &sa->quality, seq_qual_len);
     if (seq_qual_len != NULL && sa->quality != NULL) {
         sa->n_quality = *arr_len;
     }
     if (DICT_GET_OBJ(p_obj, sa_dict, "SEQUENCE_PRIMER_PAIR_OK_REGION_LIST")){
-        if ((p_obj2 = PySequence_Fast(p_obj, "Value of 'SEQUENCE_PRIMER_PAIR_OK_REGION_LIST' " 
-                                   "must support the seqeunce protocol.")) == NULL){                          
-            return NULL;}                                                  
-        sa->ok_regions.count = 0;                                                       
-        sa->ok_regions.any_pair = 0;                                                      
-        sa->ok_regions.any_left = sa->ok_regions.any_right = 0;                                   
-        len1 = (int)PySequence_Fast_GET_SIZE(p_obj2);  
-        for (i = 0; i < len1; i++) {   
+        if ((p_obj2 = PySequence_Fast(p_obj, "Value of 'SEQUENCE_PRIMER_PAIR_OK_REGION_LIST' "
+                                   "must support the seqeunce protocol.")) == NULL){
+            return NULL;}
+        sa->ok_regions.count = 0;
+        sa->ok_regions.any_pair = 0;
+        sa->ok_regions.any_left = sa->ok_regions.any_right = 0;
+        len1 = (int)PySequence_Fast_GET_SIZE(p_obj2);
+        for (i = 0; i < len1; i++) {
             p_obj3 = PySequence_Fast_GET_ITEM(p_obj, (Py_ssize_t)i);
             if ((p_obj4 = PySequence_Fast(p_obj3, "'SEQUENCE_PRIMER_PAIR_OK_REGION_LIST' must support "
                                                   "the sequence protocol and be comprised of items "
                                                   "that support the sequence protocol (e.g., it must be "
                                                   "a list of lists, tuple of tuples or some combination "
-                                                  "of the two).")) == NULL){                          
+                                                  "of the two).")) == NULL){
                 return NULL;
-            } 
-            len2 = (int)PySequence_Fast_GET_SIZE(p_obj4);              
-            if (!len2 == 4) {                                                   
-                PyErr_Format(PyExc_TypeError, "Sub-list/tuple #%d of " 
+            }
+            len2 = (int)PySequence_Fast_GET_SIZE(p_obj4);
+            if (!len2 == 4) {
+                PyErr_Format(PyExc_TypeError, "Sub-list/tuple #%d of "
                              "'SEQUENCE_PRIMER_PAIR_OK_REGION_LIST' must "
-                             "of length 4", i);  
-                return NULL;                                                      
-            }                                                                      
-            for (j = 0; j < 4; j++) { 
+                             "of length 4", i);
+                return NULL;
+            }
+            for (j = 0; j < 4; j++) {
                 if (!PyLong_Check(PySequence_Fast_GET_ITEM(p_obj4, (Py_ssize_t)j))) {
-                    PyErr_Format(PyExc_TypeError, "Object #%d of sub-list/tuple %d" 
+                    PyErr_Format(PyExc_TypeError, "Object #%d of sub-list/tuple %d"
                          "'SEQUENCE_PRIMER_PAIR_OK_REGION_LIST' must "
-                         "be an integer or long", j, i);  
+                         "be an integer or long", j, i);
                     return NULL;
-                }        
-            }                     
-            p3_add_to_2_interval_array(&sa->ok_regions,                                   
-                   (int)PyLong_AsLong(PySequence_Fast_GET_ITEM(p_obj4, (Py_ssize_t)0)),              
-                   (int)PyLong_AsLong(PySequence_Fast_GET_ITEM(p_obj4, (Py_ssize_t)1)),            
-                   (int)PyLong_AsLong(PySequence_Fast_GET_ITEM(p_obj4, (Py_ssize_t)2)),            
-                   (int)PyLong_AsLong(PySequence_Fast_GET_ITEM(p_obj4, (Py_ssize_t)3)));             
-        }  
-    }                
+                }
+            }
+            p3_add_to_2_interval_array(&sa->ok_regions,
+                   (int)PyLong_AsLong(PySequence_Fast_GET_ITEM(p_obj4, (Py_ssize_t)0)),
+                   (int)PyLong_AsLong(PySequence_Fast_GET_ITEM(p_obj4, (Py_ssize_t)1)),
+                   (int)PyLong_AsLong(PySequence_Fast_GET_ITEM(p_obj4, (Py_ssize_t)2)),
+                   (int)PyLong_AsLong(PySequence_Fast_GET_ITEM(p_obj4, (Py_ssize_t)3)));
+        }
+    }
     DICT_GET_AND_COPY_TO_INTERVAL_ARRAY(p_obj, sa_dict, "SEQUENCE_TARGET", sa->tar2);
     DICT_GET_AND_COPY_TO_INTERVAL_ARRAY(p_obj, sa_dict, "SEQUENCE_EXCLUDED_REGION", sa->excl2);
     DICT_GET_AND_COPY_TO_INTERVAL_ARRAY(p_obj, sa_dict, "SEQUENCE_INTERNAL_EXCLUDED_REGION", sa->excl_internal2);
-    DICT_GET_AND_COPY_ARRAY_INTO_ARRAY(p_obj, sa_dict,  "SEQUENCE_OVERLAP_JUNCTION_LIST", &sa->primer_overlap_junctions, overlap_junction_len);
-    if (overlap_junction_len != NULL && sa->primer_overlap_junctions != NULL) {
-        sa->primer_overlap_junctions_count = *overlap_junction_len;
+    // DICT_GET_AND_COPY_ARRAY_INTO_ARRAY(p_obj, sa_dict,  "SEQUENCE_OVERLAP_JUNCTION_LIST", &sa->primer_overlap_junctions, overlap_junction_len);
+    int overlap_junction_arr_len = 0;
+    if (DICT_GET_OBJ(p_obj, sa_dict, "SEQUENCE_OVERLAP_JUNCTION_LIST")) {
+        if (!PyList_Check(p_obj)){
+            PyErr_Format(PyExc_TypeError,
+                            "Value of 'SEQUENCE_OVERLAP_JUNCTION_LIST' is not of type list");
+            return NULL;}
+        overlap_junction_arr_len = (int)PyList_Size(p_obj);
+        if (overlap_junction_arr_len > 200) {
+            PyErr_Format(PyExc_TypeError,
+                            "'SEQUENCE_OVERLAP_JUNCTION_LIST' cannot have over 200 values");
+            return NULL;
+        }
+        sa->primer_overlap_junctions_count = overlap_junction_arr_len;
+        int *poj_arr = &sa->primer_overlap_junctions[0];
+        for (i=0; i < overlap_junction_arr_len; i++, poj_arr++) {
+            *poj_arr = (int)PyLong_AsLong(PyList_GetItem(p_obj, i));
+        }
     }
     if DICT_GET_OBJ(p_obj, sa_dict, "SEQUENCE_INCLUDED_REGION") {
         if (!PyList_Check(p_obj)) {
